//=============================================================================
// DataManager.cpp
//=============================================================================
#include "Stdinc.h"
#include "DataManager.h"
#include "MapManager.h"
#include "MapObject.h"
#include "MapEvent.h"
#include "CSVString.h"
#include "Item.h"

const MapDataInfo MAP_DATA_INFO[] = 
{
	{ 000001,	// eXg}bv1 (X)
		FALSE,
		{  10002,  10002,  10002 },
	},
	{ 000002, 	// eXg}bv2 (tB[h)
		TRUE,
		{ 900001, 900003, 900003 },
	},
	{ 000003,	// eXg}bv2 (X)
		FALSE,
		{ 800001, 800002, 800003 },
	},
};
//-----------------------------------------------------------------------------
// RXgN^
//-----------------------------------------------------------------------------
CDataManager::CDataManager(void)
:m_bLoadedMapChipGraph( FALSE ), m_bLoadedCharacterGraph( FALSE ), m_bLoadedFrameWndGraph( FALSE )
{
}

//-----------------------------------------------------------------------------
// fXgN^
//-----------------------------------------------------------------------------
CDataManager::~CDataManager(void)
{

}
//-----------------------------------------------------------------------------
// GetInstance
//-----------------------------------------------------------------------------
CDataManager *CDataManager::GetInstance( void )
{
	static CDataManager m_Instance;
	return &m_Instance;
}
//-----------------------------------------------------------------------------
// LoadMap
//-----------------------------------------------------------------------------
BOOL CDataManager::LoadMap( int nMapID, CMapManager *pMap )
{
	char szMapFileName[MAX_PATH];
	RtlZeroMemory( szMapFileName, sizeof(szMapFileName) );


	// IDΉ }bv擾
	int nChipID[LAYER_CHIP_MAX];	// CChipG
	int nIndex = 0;
	for ( int iCnt = 0; iCnt < sizeof(MAP_DATA_INFO) / sizeof(MAP_DATA_INFO[0]); iCnt ++ )
	{
		if ( MAP_DATA_INFO[iCnt].nMapID == nMapID )
		{
			nIndex = iCnt;
			// }bvt@C
			sprintf_s( szMapFileName,	 "%s\\map%06d.dat",  MAP_DATA_DIR, MAP_DATA_INFO[iCnt].nMapID );
			// }bv`bvID
			memcpy_s( nChipID, sizeof(nChipID),  MAP_DATA_INFO[iCnt].nChipID, sizeof(nChipID) );
			break;
		}
	}

	// }bvt@CǍ
	int nFileHandle = FileRead_open( szMapFileName );
	if ( nFileHandle == DX_INVALID_HANDLE )
	{
		return FALSE;
	}

	// wb_Ǎ
	FMFHEADER fmfHeader;
	if ( FileRead_read( &fmfHeader, sizeof(fmfHeader), nFileHandle ) == DX_FAILED )
	{
		FileRead_close( nFileHandle );
		return FALSE;
	}

	// t@C
	if ( fmfHeader.byLayerCount != LAYER_MAX )
	{
		FileRead_close( nFileHandle );
		return FALSE;
	}

	// }bvݒ
	pMap->SetMapInfo( fmfHeader.dwWidth, fmfHeader.dwHeight, nChipID, MAP_DATA_INFO[nIndex].bLoopAtEdge );

	// `bv̈m
	WORD **pMapChip = pMap->GetMapChip();
	int iReadCnt = fmfHeader.dwWidth * fmfHeader.dwHeight;
	for ( int iLayer = 0; iLayer < LAYER_CHIP_MAX; iLayer ++ )
	{
		pMapChip[iLayer] = new WORD[iReadCnt];
		RtlZeroMemory( pMapChip[iLayer], sizeof(WORD) * iReadCnt );
		if ( FileRead_read( pMapChip[iLayer], sizeof(WORD) * iReadCnt, nFileHandle ) == DX_FAILED )
		{
			FileRead_close( nFileHandle );
			return FALSE;
		}
	}

	// ʍs/s擾
	pMap->m_wPassFlag = new WORD[iReadCnt];
	RtlZeroMemory( pMap->m_wPassFlag, sizeof(WORD) * iReadCnt );
	if ( FileRead_read( pMap->m_wPassFlag, sizeof(WORD) * iReadCnt, nFileHandle ) == DX_FAILED )
	{
		FileRead_close( nFileHandle );
		return FALSE;
	}

	// }bvIuWFNg擾
	pMap->m_MapEvent.m_wMapObject = new WORD[iReadCnt];
	RtlZeroMemory( pMap->m_MapEvent.m_wMapObject, sizeof(WORD) * iReadCnt );
	if ( FileRead_read( pMap->m_MapEvent.m_wMapObject, sizeof(WORD) * iReadCnt, nFileHandle ) == DX_FAILED )
	{
		FileRead_close( nFileHandle );
		return FALSE;
	}

	FileRead_close( nFileHandle );

	return TRUE;
}


//-----------------------------------------------------------------------------
// LoadMapChipGraph
// S}bv`bvǍ
//-----------------------------------------------------------------------------
BOOL CDataManager::LoadMapChipGraph( std::map<int, int> &mapInfo )
{
	if ( m_bLoadedMapChipGraph )
	{
		return TRUE;
	}

	using namespace std;

	char szFileName[MAX_PATH];
	for ( int iCnt = 0; iCnt < MAP_CHIP_GRAPH_MAX; iCnt ++ )
	{
		sprintf_s( szFileName, sizeof(szFileName), "%s\\chip%06d.bmp", MAP_DATA_DIR, MAP_CHIP_ID[iCnt] );

		// }bv`bvGǂݍ
		SetTransColor(0,117,117);
		m_nHandleMapChip[iCnt] = ::LoadGraph( szFileName );
		if ( m_nHandleMapChip[iCnt] == DX_FAILED )
		{
			CSysError::Msg("摜ǂݍݎs [%s]", szFileName );
			return FALSE;
		}

		mapInfo.insert( pair<int, int>( MAP_CHIP_ID[iCnt], m_nHandleMapChip[iCnt] ));
	}

	m_bLoadedMapChipGraph = TRUE;

	return TRUE;
}
//-----------------------------------------------------------------------------
// DeleteMapChipGraph
// S}bv`bv폜
//-----------------------------------------------------------------------------
void CDataManager::DeleteMapChipGraph( void )
{
	for ( int iCnt = 0; iCnt < MAP_CHIP_GRAPH_MAX; iCnt ++ )
	{
		DeleteGraph( m_nHandleMapChip[iCnt] );
	}
	m_bLoadedMapChipGraph = FALSE;

}
//-----------------------------------------------------------------------------
// LoadCharacterGraph
// SLN^GǍ
//-----------------------------------------------------------------------------
BOOL CDataManager::LoadCharacterGraph( void )
{
	if ( m_bLoadedCharacterGraph )
	{
		return TRUE;
	}

	char szFileName[MAX_PATH];
	for ( int iCnt = 0; iCnt < CHAR_GRAPH_MAX; iCnt ++ )
	{
		sprintf_s( szFileName, sizeof(szFileName), "%s\\%s%02d.png", CHAR_DATA_DIR, CHAR_GRAPH_FNAME_PREFIX, iCnt+1 );

		// Gǂݍ
		SetTransColor(0,117,117);
		m_nHandleCharacter[iCnt] = ::LoadGraph( szFileName );
		if ( m_nHandleCharacter[iCnt] == DX_FAILED )
		{
			CSysError::Msg("摜ǂݍݎs [%s]", szFileName );
			return FALSE;
		}
	}
	m_bLoadedCharacterGraph = TRUE;

	return TRUE;
}
//-----------------------------------------------------------------------------
// DeleteMapChipGraph
// SLN^G폜
//-----------------------------------------------------------------------------
void CDataManager::DeleteCharacterGraph( void )
{
	for ( int iCnt = 0; iCnt < CHAR_GRAPH_MAX; iCnt ++ )
	{
		DeleteGraph( m_nHandleCharacter[iCnt] );
	}

	m_bLoadedCharacterGraph = FALSE;

}

//-----------------------------------------------------------------------------
// GetMapObjectLocation
//-----------------------------------------------------------------------------
void CDataManager::GetMapObjectLocation( int *pX, int *pY, int nEventID, const CMapEvent *pMapEvent )
{
	for ( int iCnt = 0; iCnt < pMapEvent->m_nWidth * pMapEvent->m_nHeight; iCnt ++ )
	{
		if ( pMapEvent->m_wMapObject[iCnt] == nEventID )
		{
			*pX = iCnt % pMapEvent->m_nWidth;
			*pY = iCnt / pMapEvent->m_nWidth;
		}
	}
}
//-----------------------------------------------------------------------------
// LoadOneItemData
//-----------------------------------------------------------------------------
BOOL CDataManager::LoadOneItemData( int nItemID, CItem **pItem )
{
	// ̈m
	if ( CEquipItem::IsEquipItem( nItemID ) )
	{
		*pItem  = new CEquipItem();
	}
	else
	{
		*pItem  = new CItem();
	}

	// ACe
	if ( nItemID < ITEM_ID_BASE_WEAPON )
	{
		// ʏACe
		return LoadOneNormalItemDataCSV( nItemID, *pItem );	// JCSVǂ
	}
	else if ( nItemID < ITEM_ID_BASE_ACCESSORY )
	{	// /h
		return LoadOneEquipItemDataCSV( nItemID, (CEquipItem *)*pItem );	// JCSVǂ
	}
	else
	{	// ANZT

	}

	return FALSE;
}
//-----------------------------------------------------------------------------
// LoadOneNormalItemDataCSV
// Ĵݎgp
//-----------------------------------------------------------------------------
BOOL CDataManager::LoadOneNormalItemDataCSV( int nItemID, CItem *pItem )
{
	char szLine[32*1024];
	char szFileName[MAX_PATH];
	char wkBuff[1024];
	FILE *fp;

	sprintf_s( szFileName, sizeof(szFileName), "%s\\%s", ITEM_DATA_DIR, ITEM_NORMAL_CSV_FILE_NAME );
	//  Item.csv
	if ( fopen_s( &fp, szFileName, "r" ) != 0 )
	{
		CSysError::Msg("t@Cǂݍݎs [%s]", szFileName);
		return FALSE;
	}
	// 1,2sڂ xs
	fgets( szLine, sizeof(szLine), fp );
	fgets( szLine, sizeof(szLine), fp );

	// 3sڂf[^
	BOOL bFound = FALSE;
	CItem ItemData;
	RtlZeroMemory( &ItemData, sizeof(ItemData) );

	while ( fgets( szLine, sizeof(szLine), fp ) )
	{
		LPCSTR pPtr = szLine;

		// ID
		ItemData.nID = CCSVString::GetNextInt( pPtr );
		if ( nItemID != ItemData.nID )
		{
			continue;
		}

		// 
		bFound = TRUE;

		// O
		CCSVString::GetNextString( wkBuff, sizeof(wkBuff), pPtr );
		strcpy_s( ItemData.szName, wkBuff );

		// l
		ItemData.nValue = CCSVString::GetNextInt( pPtr );

		// 
		CCSVString::GetNextString( wkBuff, sizeof(wkBuff), pPtr );
		strcpy_s( ItemData.szDesc, wkBuff );

		break;

	}
	fclose( fp );

	if ( !bFound )
	{	return FALSE; }

	// ACeRs[
	*pItem = ItemData;

	return TRUE;
}

//-----------------------------------------------------------------------------
// LoadOneEquipItemDataCSV
// Ĵݎgp
//-----------------------------------------------------------------------------
BOOL CDataManager::LoadOneEquipItemDataCSV( int nItemID, CEquipItem *pItem )
{
	char szLine[32*1024];
	char szFileName[MAX_PATH];
	char wkBuff[1024];
	FILE *fp;

	if ( nItemID < ITEM_ID_BASE_ARMOR )
	{	// 
		sprintf_s( szFileName, sizeof(szFileName), "%s\\%s", ITEM_DATA_DIR, ITEM_WEAPON_CSV_FILE_NAME );
	}
	else
	{	// h
		sprintf_s( szFileName, sizeof(szFileName), "%s\\%s", ITEM_DATA_DIR, ITEM_ARMOR_CSV_FILE_NAME );
	}
	//  Item.csv
	if ( fopen_s( &fp, szFileName, "r" ) != 0 )
	{
		CSysError::Msg("t@Cǂݍݎs [%s]", szFileName);
		return FALSE;
	}
	// 1,2sڂ xs
	fgets( szLine, sizeof(szLine), fp );
	fgets( szLine, sizeof(szLine), fp );

	// 3sڂf[^
	BOOL bFound = FALSE;
	CEquipItem ItemData;
	RtlZeroMemory( &ItemData, sizeof(ItemData) );

	while ( fgets( szLine, sizeof(szLine), fp ) )
	{
		LPCSTR pPtr = szLine;

		// ID
		ItemData.nID = CCSVString::GetNextInt( pPtr );
		if ( nItemID != ItemData.nID )
		{
			continue;
		}

		// 
		bFound = TRUE;

		// O
		CCSVString::GetNextString( wkBuff, sizeof(wkBuff), pPtr );
		strcpy_s( ItemData.szName, wkBuff );
		// l
		ItemData.nValue = CCSVString::GetNextInt( pPtr );
		// 
		CCSVString::GetNextString( wkBuff, sizeof(wkBuff), pPtr );
		strcpy_s( ItemData.szDesc, wkBuff );

		// 
		ItemData.bEquiped = FALSE;
		// 
		ItemData.nType = CCSVString::GetNextInt( pPtr );
		// ʒu
		ItemData.nEquipPos = CCSVString::GetNextInt( pPtr );

		// U
		ItemData.Param.nAttack = CCSVString::GetNextInt( pPtr );
		// h
		ItemData.Param.nDefence = CCSVString::GetNextInt( pPtr );
		// @h
		ItemData.Param.nMagicDef = CCSVString::GetNextInt( pPtr );
		// 
		ItemData.Param.nParry = CCSVString::GetNextInt( pPtr );
		// @
		ItemData.Param.nMagicPar = CCSVString::GetNextInt( pPtr );
		// d
		ItemData.Param.nWeight = CCSVString::GetNextInt( pPtr );
		//  (%)
		ItemData.Param.nHitRate = CCSVString::GetNextInt( pPtr );
		// MAX HP
		ItemData.BaseParam.nMaxHP = CCSVString::GetNextInt( pPtr );
		// MAX MP
		ItemData.BaseParam.nMaxMP = CCSVString::GetNextInt( pPtr );
		// 
		ItemData.BaseParam.nStrength = CCSVString::GetNextInt( pPtr );
		// ̗
		ItemData.BaseParam.nVitality = CCSVString::GetNextInt( pPtr );
		// f
		ItemData.BaseParam.nSpeed = CCSVString::GetNextInt( pPtr );
		// 
		ItemData.BaseParam.nIntellect = CCSVString::GetNextInt( pPtr );
		// _
		ItemData.BaseParam.nPiety = CCSVString::GetNextInt( pPtr );
		// ^
		ItemData.BaseParam.nLuck = CCSVString::GetNextInt( pPtr );

		break;

	}
	fclose( fp );

	if ( !bFound )
	{	return FALSE; }

	// ACeRs[
	*pItem = ItemData;

	return TRUE;
}
//-----------------------------------------------------------------------------
// LoadFrameWndGraph
//-----------------------------------------------------------------------------
BOOL CDataManager::LoadFrameWndGraph( void )
{
	// ɓǂݍݍς݂
	if ( m_bLoadedFrameWndGraph )
	{
		return TRUE;
	}

	// t[Gǂݍ
	int iRet = LoadDivGraph( WINDOW_FRAME_CHIPS_FILE_NAME , 8 , 9 , 1 , FRAME_CHIP_ONE_SIDE_SIZE, FRAME_CHIP_ONE_SIDE_SIZE , m_nHandleFrameWnd ) ;
	if ( iRet != DX_SUCCESS )
	{
		CSysError::Msg("摜ǂݍݎs [%s]", WINDOW_FRAME_CHIPS_FILE_NAME);
		return FALSE;
	}

	// wi쐬
	const int BACK_GROUND = 8;
	m_nHandleFrameWnd[ BACK_GROUND ] = MakeGraph( CANVAS_WIDTH, CANVAS_HEIGHT ) ;
	// `ύX
	SetDrawScreen( m_nHandleFrameWnd[ BACK_GROUND ] );

	const int nThickness = 8;	// 1Linȇ
	const int nInitR = 128;
	const int nInitG = 128;
	const int nInitB = 255;
	for ( int iCnt = 0; iCnt < CANVAS_HEIGHT; iCnt += nThickness )
	{	// FvZ

		int nR,nG,nB;
		// œ삪ς
		if ( iCnt < (CANVAS_HEIGHT/2) )
		{	// ԗ 128 -> 0
			nR = nInitR * (CANVAS_HEIGHT/2 - iCnt) / (CANVAS_HEIGHT/2);
			nG = nInitG * (CANVAS_HEIGHT/2 - iCnt) / (CANVAS_HEIGHT/2);
			nB = nInitB;
		}
		else
		{	//   255 -> 0
			nR = nG = 0;
			nB = nInitB - ((iCnt + nThickness) - CANVAS_HEIGHT/2);
		}
		// C`
		DWORD dwColor = GetColor( nR, nG, nB );
		DrawLine( 0, iCnt, CANVAS_WIDTH, iCnt, dwColor, nThickness );
	}
	// `߂
	SetDrawScreen( DX_SCREEN_BACK );

	return TRUE;
}
//-----------------------------------------------------------------------------
// DeleteFrameWndGraph
// St[EBhEG폜
//-----------------------------------------------------------------------------
void CDataManager::DeleteFrameWndGraph( void )
{
	for ( int iCnt = 0; iCnt < FRAME_CHIP_COUNT; iCnt ++ )
	{
		if ( m_nHandleFrameWnd[iCnt] )
		{
			DeleteGraph( m_nHandleFrameWnd[iCnt] );
		}
	}
	m_bLoadedFrameWndGraph = FALSE;
}



//-----------------------------------------------------------------------------
// LoadLvInfoDataCSV
// Ĵݎgp
//-----------------------------------------------------------------------------
BOOL CDataManager::LoadLvInfoDataCSV( int nPlayerCharID, int nLv, BASE_PARAM *pBaseParam )
{
	char szLine[32*1024];
	char szFileName[MAX_PATH];
	FILE *fp;

	sprintf_s( szFileName, sizeof(szFileName), "%s\\%s%03d.csv", LV_INFO_DATA_DIR, LV_INFO_DATA_FNAME_PREFIX, nPlayerCharID );

	//  Item.csv
	if ( fopen_s( &fp, szFileName, "r" ) != 0 )
	{
		CSysError::Msg("t@Cǂݍݎs [%s]", szFileName);
		return FALSE;
	}
	// 1,2sڂ xs
	fgets( szLine, sizeof(szLine), fp );
	fgets( szLine, sizeof(szLine), fp );

	// 3sڂf[^
	BOOL bFound = FALSE;
	BASE_PARAM BaseParam;
	RtlZeroMemory( &BaseParam, sizeof(BaseParam) );

	while ( fgets( szLine, sizeof(szLine), fp ) )
	{
		LPCSTR pPtr = szLine;

		// Lv
		int nWkLv = CCSVString::GetNextInt( pPtr );
		if ( nWkLv != nLv )
		{
			continue;
		}
		// 
		bFound = TRUE;

		// Kvol(ݐ)
		int nExp = CCSVString::GetNextInt( pPtr );

		// MaxHP
		BaseParam.nMaxHP = CCSVString::GetNextInt( pPtr );
		// MAX MP
		BaseParam.nMaxMP = CCSVString::GetNextInt( pPtr );
		// 
		BaseParam.nStrength = CCSVString::GetNextInt( pPtr );
		// ̗
		BaseParam.nVitality = CCSVString::GetNextInt( pPtr );
		// f
		BaseParam.nSpeed = CCSVString::GetNextInt( pPtr );
		// 
		BaseParam.nIntellect = CCSVString::GetNextInt( pPtr );
		// _
		BaseParam.nPiety = CCSVString::GetNextInt( pPtr );
		// ^
		BaseParam.nLuck = CCSVString::GetNextInt( pPtr );

		break;

	}
	fclose( fp );

	if ( !bFound )
	{	return FALSE; }

	// ACeRs[
	*pBaseParam = BaseParam;

	return TRUE;
}

//-----------------------------------------------------------------------------
// GetToNextLvCSV
// Ĵݎgp
//-----------------------------------------------------------------------------
int CDataManager::GetToNextLvCSV( int nPlayerCharID, int nLv, int nExp )
{
	char szLine[32*1024];
	char szFileName[MAX_PATH];
	FILE *fp;

	int nRetExp = 0;

	sprintf_s( szFileName, sizeof(szFileName), "%s\\%s%03d.csv", LV_INFO_DATA_DIR, LV_INFO_DATA_FNAME_PREFIX, nPlayerCharID );

	//  Item.csv
	if ( fopen_s( &fp, szFileName, "r" ) != 0 )
	{
		CSysError::Msg("t@Cǂݍݎs [%s]", szFileName);
		return FALSE;
	}
	// 1,2sڂ xs
	fgets( szLine, sizeof(szLine), fp );
	fgets( szLine, sizeof(szLine), fp );

	// 3sڂf[^
	BOOL bFound = FALSE;
	BASE_PARAM BaseParam;
	RtlZeroMemory( &BaseParam, sizeof(BaseParam) );

	while ( fgets( szLine, sizeof(szLine), fp ) )
	{
		LPCSTR pPtr = szLine;

		// Lv
		int nWkLv = CCSVString::GetNextInt( pPtr );
		if ( nWkLv != nLv + 1 )
		{
			continue;
		}
		// 
		bFound = TRUE;

		// Kvol(ݐ)
		int nWkExp = CCSVString::GetNextInt( pPtr );
		nRetExp = nWkExp - nExp;
		if ( nRetExp < 0 )
			nRetExp = 0;
		break;

	}
	fclose( fp );

	if ( !bFound )
	{	return 0; }

	return nRetExp;
}
//-----------------------------------------------------------------------------
// GetCharName
//-----------------------------------------------------------------------------
const char *CDataManager::GetCharName( int nPlayerCharID ) const
{
	switch ( nPlayerCharID )
	{
	case 0:
		return "";
	case 1:
		return "fBm";
	case 2:
		return "}A";
	case 3:
		return "~A";
	case 4:
		return "WbNX";
	}
	return "";
}


//  Wu֘A
// ToDo : ̃NXɉu
//-----------------------------------------------------------------------------
// GetJobName
//-----------------------------------------------------------------------------
const char *CDataManager::GetJobName( int nID )
{
	switch ( nID )
	{
	case JOB_ID_SUPPIN:
		return "҂";

	case JOB_ID_KNIGHT:
		return "";

	case JOB_ID_BLACK_WIZARD:
		return "܂ǂ";

	case JOB_ID_WHITE_WIZARD:
		return "܂ǂ";

	default:
		break;
	}
	return "ӂ߂";
}
